# -*- coding: utf-8 -*-
# pylint: disable=missing-docstring,unused-import,reimported
import pathlib

import pytest
from git import Repo

import foran.foran as fb
from foran.status import Status


def test_main():
    assert fb.main(['foran-eller-bagved.functional']) == 0


def test_local_commits():
    repo = Repo('.')
    status = Status(repo)
    status.foran = False
    fb.local_commits(repo, status)
    assert status.local_commits == []


def test_local_commits_no_remote():
    path_to_non_remote = pathlib.Path('tests', 'fixtures', 'non_remote')
    repo = Repo(path_to_non_remote)
    status = Status(repo)
    status.foran = False
    message = 'No remote found, so all commit differences hypothetical'
    with pytest.raises(UserWarning) as ex:
        fb.local_commits(repo, status)
        assert status.local_commits == [message]
        assert message in str(ex.value)



# -*- coding: utf-8 -*-
# pylint: disable=expression-not-assigned,line-too-long
"""In front or behind (Foran eller bagved)? reporting interface."""
import pathlib
from enum import Enum, auto
from typing import Tuple

from foran.status import Status

REPORT_STEM = 'foran-eller-bagved'


class Format(Enum):
    NONE = auto()
    TEXT = auto()


class Report:
    """Report structure."""

    def __init__(self, stem: str = REPORT_STEM, file_format: Format = Format.TEXT):
        """Seed the status structure with the git status info and some defaults."""
        self.stem = stem
        self.file_format = file_format


def report_as(status: Status, report: Report) -> None:
    """Side effects ..."""
    file_extension = '.txt' if report.file_format == Format.TEXT else ''  # TODO(sthagen) HACK A DID ACK
    filepath = pathlib.Path(f'{report.stem}{file_extension}')
    filepath.parent.mkdir(parents=True, exist_ok=True)

    with open(filepath, 'w') as handle:
        handle.write(''.join(generate_report(status)))


def generate_report(status: Status) -> Tuple[str, ...]:
    """Convoluted special trickery ... to build partially conditional report lines"""
    report = []
    report.append(f'Analysis ({status.when})\n')
    report.append(f'State    ({status.foran_disp})\n')
    report.append(f'Branch   ({status.branch})\n')
    report.append(f'Commit   ({status.commit})\n')

    if not status.foran:
        report.append('List of local commits:\n')
        report.append(''.join(f' - {commit}\n' for commit in status.local_commits))

    if status.local_files:
        report.append('List of locally modified files: \n')
        report.append(''.join(f' - {mod_file}\n' for mod_file in status.local_files))

    return tuple(report)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# pylint: disable=line-too-long
"""Commandline API gateway for foran."""
import sys
from typing import List, Union

import foran.foran as fb


# pylint: disable=expression-not-assigned
def main(argv: Union[List[str], None] = None) -> int:
    """Delegate processing to functional module."""
    argv = sys.argv[1:] if argv is None else argv
    return fb.main(argv)



# -*- coding: utf-8 -*-
# pylint: disable=expression-not-assigned,line-too-long
"""In front or behind (Foran eller bagved)? status structure."""
import datetime as dti

from git import Repo

STATUS_DTI_FORMAT = '%Y-%m-%d %H:%M:%S UTC'


class Status:
    """Status structure."""

    def __init__(self, repo: Repo, timestamp_format: str = STATUS_DTI_FORMAT):
        """Seed the status structure with the git status info and some defaults."""
        self._repo = repo
        branch_is_up_to_date = 'Your branch is up to date' in self._repo.git.status()
        self.when = dti.datetime.now(dti.timezone.utc).strftime(timestamp_format)
        self.branch = self._repo.active_branch.name
        self.foran = branch_is_up_to_date
        self.foran_disp = 'UP TO DATE' if branch_is_up_to_date else 'CUSTOM'
        self.commit = self._repo.head.commit
        self.local_commits: list[str] = []
        self.local_files: list[str] = []



# -*- coding: utf-8 -*-
# pylint: disable=expression-not-assigned,line-too-long
"""In front or behind (Foran eller bagved)? API."""
import os
import sys
import warnings
from typing import List, Union

from git import Repo
from git.exc import GitCommandError

from foran.report import Format, Report, report_as
from foran.status import Status

DEBUG_VAR = 'FORAN_DEBUG'
DEBUG = os.getenv(DEBUG_VAR)

ENCODING = 'utf-8'
ENCODING_ERRORS_POLICY = 'ignore'


def local_commits(repo: Repo, status: Status) -> None:
    """Yes"""
    if not status.foran:
        branch = repo.active_branch.name
        try:
            status.local_commits = [rev for rev in repo.iter_commits(f'origin/{branch}..{branch}')]
        except GitCommandError as ex:
            symptom = ex.stderr.replace('\n', '')
            if "fatal: bad revision 'origin/default..default" in symptom:
                warning = 'No remote found, so all commit differences hypothetical'
                status.local_commits = [warning]
                warnings.warn(warning)
            else:
                raise


def local_files(repo: Repo, status: Status) -> None:
    """Sure"""
    status.local_files = [item.a_path for item in repo.index.diff(None)]


def main(argv: Union[List[str], None] = None) -> int:
    """Drive the analysis."""
    argv = argv if argv else sys.argv[1:]
    report = Report() if not argv else Report(stem=argv[0], file_format=Format.NONE)
    repo = Repo('.')
    status = Status(repo)
    local_commits(repo, status)
    local_files(repo, status)
    report_as(status, report)
    return 0
